unit cpu940;

interface

var code940:pointer;

type  startstop=(start_clock,stop_clock);
      resetstatus=(status_operational,status_reset);

procedure cpu940_clock_startstop(b:startstop);
procedure cpu940_resetstatus(b:resetstatus);
function cpu940_getbase:byte;
procedure cpu940_setbase(base:byte);
procedure init_940(const prog:string);

implementation

uses gp2xhw,baseunix,unix;

procedure cpu940_clock_startstop(b:startstop);

begin
  if b=stop_clock then
    gp2xreg[SYSCLKENREG]:=gp2xreg[SYSCLKENREG] and $fffe
  else
    gp2xreg[SYSCLKENREG]:=gp2xreg[SYSCLKENREG] or 1;
end;

procedure cpu940_resetstatus(b:resetstatus);

begin
  if b=status_operational then
    begin
      write(gp2xreg[DUALCTRL940] and not 128);
    gp2xreg[DUALCTRL940]:=gp2xreg[DUALCTRL940] and not 128
    end
  else
    gp2xreg[DUALCTRL940]:=gp2xreg[DUALCTRL940] or 128;
end;

function cpu940_getbase:byte;

begin
  cpu940_getbase:=gp2xreg[DUALCTRL940] and $7f;
end;

procedure cpu940_setbase(base:byte);

begin
  gp2xreg[DUALCTRL940]:=gp2xreg[DUALCTRL940] and 128 or base;
end;

procedure init_940(const prog:string);

var f:file;
    fd:cint;
    r:longint;

begin
  {Stop and reset the 940.}
  cpu940_resetstatus(status_reset);
  cpu940_clock_startstop(stop_clock);
  {940 adress space start starts at physical $03000000.}
  cpu940_setbase($03);

  gp2xreg[DUALINT920]:=0;
  gp2xreg[DUALINT940]:=0;
  gp2xreg[DUALPEND920]:=$ffff;
  gp2xreg[DUALPEND940]:=$ffff;

  {Map 512k from physical memory starting at $03000000 in our address space.}
  fd:=fpopen('/dev/mem',O_RDWR);
  if fd=-1 then
    runerror(100);
  code940:=fpmmap(nil,$80000,PROT_READ or PROT_WRITE or PROT_EXEC,
            MAP_SHARED,fd,$03000000);
  if cint(code940)=-1 then
    runerror(100);
  fpclose(fd);
  {Load the 940T code into it.}
  assign(f,prog);
  reset(f,1);
  blockread(f,code940^,512*1024,r);  {Read max. 512k.}
  close(f);

  {Start the 940}
  cpu940_resetstatus(status_operational);
  cpu940_clock_startstop(start_clock);
end;

var old_base:byte;

initialization
  writeln('SYSCLKEN=',gp2xreg[SYSCLKENREG]);
  writeln('DUALCTRL940=',gp2xreg[DUALCTRL940]);
  old_base:=cpu940_getbase;
finalization
  {Make sure the 940 is always stopped on normal or abnormal program exit...}
  cpu940_resetstatus(status_reset);
  cpu940_clock_startstop(stop_clock);
  {Restore the old physical address base of the 940.}
  cpu940_setbase(old_base);
end.
