unit gbamem;

interface

procedure init_gba_mem_layout(const romfile:string);

implementation

uses baseunix,unix;

procedure init_gba_mem_layout(const romfile:string);

var fd:cint;
    p:pointer;
    s:stat;
    r:cint;

const gba_rom_protection=PROT_READ or PROT_WRITE or PROT_EXEC;

begin
  {Mmap the ROM.}
  fd:=fpopen(romfile,O_RDONLY);
  if fd=-1 then
    runerror(100);
  fpfstat(fd,s);
  p:=fpmmap(pointer($08000000),s.size,
            gba_rom_protection,MAP_FIXED or MAP_PRIVATE,fd,0);
  if cint(p)=-1 then
    if errno=EsysENODEV then
      begin
        {In case no mmap, read entire ROM into memory.}
        p:=fpmmap(pointer($08000000),s.size,
                  gba_rom_protection,MAP_FIXED or MAP_SHARED or MAP_ANONYMOUS,0,0);
        if cint(p)=-1 then
          runerror(100);
        r:=fpread(fd,pointer($08000000),s.size);
        if r=-1 then
          runerror(100);
      end
    else
      runerror(100);
  fpclose(fd);

  fd:=fpopen('/dev/mem',O_RDWR);
  if fd=-1 then
    runerror(100);


  {Create the EWRAM.}
  fpmmap(pointer($02000000),$40000,
         PROT_READ or PROT_WRITE or PROT_EXEC,
         MAP_FIXED or MAP_SHARED or MAP_ANONYMOUS,0,0);

  {Create the IWRAM.}
  fpmmap(pointer($03000000),$8000,
         PROT_READ or PROT_WRITE or PROT_EXEC,
         MAP_FIXED or MAP_SHARED or MAP_ANONYMOUS,0,0);


  {Create the custom IO area. We locate it in physical memory at address
   $03080000, so the ARM940T can access it too.}
  p:=fpmmap(pointer($04000000),$8000,PROT_READ or PROT_WRITE,
                  MAP_FIXED or MAP_SHARED,fd,$03080000);
  if cint(p)=-1 then
    runerror(100);

  {Create the palette. We locate it in physical memory at address
   $030A8000, so the ARM940T can access it too.}
  p:=fpmmap(pointer($05000000),$1000,PROT_READ or PROT_WRITE,
                  MAP_FIXED or MAP_SHARED,fd,$030A8000);
  if cint(p)=-1 then
    runerror(100);

  {Create the VRAM. We locate it in physical memory at address
   $03090000, so the ARM940T can access it too.}
  fpmmap(pointer($06000000),$18000,
         PROT_READ or PROT_WRITE or PROT_EXEC,
         MAP_FIXED or MAP_SHARED,fd,$03090000);

  {Create the OAM.}
  fpmmap(pointer($07000000),$1000,
         PROT_READ or PROT_WRITE or PROT_EXEC,
         MAP_FIXED or MAP_SHARED or MAP_ANONYMOUS,0,0);
  fpclose(fd);
end;

end.
